import { ReactWidget } from '@jupyterlab/apputils';
import { CodeCell } from '@jupyterlab/cells';
import { runIcon, stopIcon, LabIcon } from '@jupyterlab/ui-components';
import React, { useEffect, useState } from 'react';
const PlayButton = ({ icon, onClick }) => (React.createElement("button", { type: "button", onClick: () => onClick(), className: "cellPlayButton" },
    React.createElement(LabIcon.resolveReact, { icon: icon, className: "cellPlayButton-icon", tag: "span", width: "15px", height: "15px" })));
/**
 * makeCancelable
 *
 * Note: Permits a Promise to be cancelled rather than to execute a
 * chained .then callback which is necessary if the component
 * has been unmounted
 *
 * Lifted from the long running discussion here:
 * https://github.com/facebook/react/issues/5465#issuecomment-157888325
 *
 * @param promise - The Promise to make cancellable.
 */
function makeCancelable(promise) {
    let active = true;
    return {
        cancel() { active = false; },
        promise: promise.then(value => active ? value : new Promise(() => { }), reason => active ? reason : new Promise(() => { }))
    };
}
const CodeCellPlayButtonComponent = ({ cell, session, }) => {
    // A hacky way to find out if we're currently running
    // anything, but doesn't matter greatly because the status
    // will soon be updated by the returned kernel future promise
    const [isRunning, setIsRunning] = useState(!!(cell.promptNode.textContent === '[*]:'));
    const executeCell = async () => {
        CodeCell.execute(cell, session);
        setIsRunning(true);
    };
    const interruptKernel = () => {
        var _a, _b;
        void ((_b = (_a = session.session) === null || _a === void 0 ? void 0 : _a.kernel) === null || _b === void 0 ? void 0 : _b.interrupt());
    };
    useEffect(() => {
        const codeCellFuture = cell.outputArea.future;
        if (!codeCellFuture) {
            return;
        }
        const { promise, cancel } = makeCancelable(codeCellFuture.done);
        promise.then(() => {
            setIsRunning(false);
        });
        return () => {
            cancel();
        };
    }, [isRunning]);
    return (React.createElement(PlayButton, { icon: isRunning ? stopIcon : runIcon, onClick: () => (isRunning ? interruptKernel : executeCell)() }));
};
const MarkdownCellPlayButtonComponent = ({ cell }) => {
    const executeCell = () => {
        cell.rendered = true;
    };
    return (React.createElement(PlayButton, { icon: runIcon, onClick: () => executeCell() }));
};
export class CellPlayButton extends ReactWidget {
    constructor(cell, session) {
        super();
        /**
         * Constructs a new CellPlayButton widget.
         *
         * Note: Depending on the type of cell encountered
         * tries to render an appropriate play button
         * component.
         */
        this.cell = null;
        this.session = null;
        this.cell = cell;
        this.session = session;
        this.addClass('jp-CellPlayButton');
    }
    render() {
        switch (this.cell.model.type) {
            case 'markdown':
                return React.createElement(MarkdownCellPlayButtonComponent, { cell: this.cell });
            case 'code':
                return React.createElement(CodeCellPlayButtonComponent, { cell: this.cell, session: this.session });
            default:
                break;
        }
    }
}
